import java.util.ArrayList;
import java.util.List;
import jp.hozo.core.Concept;
import jp.hozo.core.RoleHolder;

public class ConceptTreeNode {
	
	private Concept concept;
	private List<String> valueList = new ArrayList<String>();
	private boolean isMerged = false;

	private ConceptTreeNode parent = null;
	private List<ConceptTreeNode> children = new ArrayList<ConceptTreeNode>();
	
	private List<ConceptTreeNode> mergedNodes;
	private List<String> mergedStrings;
	
	private ConceptTreeNode representNode;

	public ConceptTreeNode() {
	}
	
	public ConceptTreeNode( Concept concept, List<String> valueList ) {
		this.concept = concept;
		this.valueList = valueList;
	}
	
	public Concept getConcept() {
		return concept;
	}

	public void setConcept(Concept concept) {
		this.concept = concept;
	}

	public List<String> getValueList() {
		return valueList;
	}

	public void setValueList(List<String> valueList) {
		this.valueList = valueList;
	}
	
	public List<ConceptTreeNode> getChildren()	{
		return children;
	}
	
	public void addChild(ConceptTreeNode node) {
		if (!this.getChildren().contains(node)) {
			this.getChildren().add(node);
			node.setParent(this);
		}
	}
	
	public ConceptTreeNode getParent()	{
		return parent;
	}
	
	private void setParent(ConceptTreeNode node)	{
		this.parent = node;
	}
	
	public String toString() {
		String label = "";
		if (concept != null) {
			if (concept instanceof RoleHolder) {
				label = ((RoleHolder) concept).getRoleHolderLabel();
			} else {
				label = concept.getLabel();
			}
		}
		for (String value : valueList) {
			if (valueList.size() > 1) {
				label = label + value + ", ";
			} else {
				label = value;
			}
		}
		return "ConceptTreeNode: " + label + ", " + valueList.toString();
	}

	public void setChildren(List<ConceptTreeNode> children) {
		this.children = children;
		
		for (ConceptTreeNode child : children) {
			child.setParent(this);
		}
	}
	
	public boolean isMerged() {
		return isMerged;
	}

	public void setMerged(boolean isMerged) {
		this.isMerged = isMerged;
		
		if (isMerged) {
			mergedNodes = new ArrayList<ConceptTreeNode>(getChildren());
			mergeValue();
			children.clear();
		} else {
			setChildren(mergedNodes);
			unmergeValue();
		}
	}
	
	public ConceptTreeNode getRepresentNode() {
		return representNode;
	}

	public void setRepresentNode(ConceptTreeNode representNode) {
		this.representNode = representNode;
	}
	
	private void mergeValue() {
		if (representNode != null) {
			List<String> curValueList = this.getValueList();
			mergedStrings = new ArrayList<String>(curValueList);
			List<String> repValueList = representNode.getValueList();
			valueList = new ArrayList<String>(repValueList);
		}
	}
	
	private void unmergeValue() {
		valueList = mergedStrings;
	}
}
